#======================================================================
# Define options
#======================================================================
set val(chan)       	Channel/WirelessChannel
set val(prop)       	Propagation/TwoRayGround
set val(netif)      	Phy/WirelessPhy
set val(mac)        	Mac/802_11
set val(ifq)        	Queue/DropTail/PriQueue
set val(ifqlen)         50            	;# max packet in ifq
set val(adhocRouting)   DSDV  
set val(ll)         	LL
set val(ant)        	Antenna/OmniAntenna
set val(x)              500   			;# X dimension of the topography
set val(y)              500   			;# Y dimension of the topography
set val(seed)           0.0
#set val(nn)			2
set val(nn)				1
set val(mainFTP-start)	40
set val(mainFTP-stop)	10
set val(crossCBR-start)	10
set val(crossCBR-stop)	1500
set val(stop)   		1500             ;# time to stop simulation
set val(tr)          	dtnTestWireless.tr
set val(namtr)       	dtnTestWireless.nam

set num_wired_nodes		4
set num_bs_nodes		1
set transferSize 		1000000
set tcpPacketSize		1460

Mac/802_11 	set dataRate_ 11Mb
Mac/802_11 	set basicRate_ 1Mb
Agent/TCP 	set window_ 100

# ======================================================================
# Main Program
# ======================================================================
set ns_ [new Simulator]

$ns_ color 1 Green
$ns_ color 2 Blue

# set up for hierarchical routing
$ns_ node-config -addressType hierarchical 3 8 8 8

AddrParams set domain_num_ 2			;# number of domains
lappend cluster_num 4 1					;# number of clusters in each domain
AddrParams set cluster_num_ $cluster_num

lappend eilastlevel 1 1 1 1 2
AddrParams set nodes_num_ $eilastlevel	;# of each cluster

set namtracefd [open $val(namtr) w]
$ns_ namtrace-all $namtracefd

set topo [new Topography]
$topo load_flatgrid $val(x) $val(y)
# god needs to know the number of all wireless interfaces
create-god [expr $val(nn) + $num_bs_nodes]

# Wired part
set temp {0.0.0 0.1.0 0.2.0 0.3.0}
for {set i 0} {$i < $num_wired_nodes} {incr i} {
	set wiredNode($i) [$ns_ node [lindex $temp $i]]
	$wiredNode($i) label "Relaying DTN"
}

$wiredNode(0) shape hexagon
$wiredNode(0) label "Source DTN"
$wiredNode(0) label-at "Down"

Trace set show_dtnhdr_ 1

set tracefd  [open $val(tr) w]
$ns_ trace-all $tracefd

# Wireless part
$ns_ node-config	-adhocRouting $val(adhocRouting) \
					-llType $val(ll) \
					-macType $val(mac) \
					-ifqType $val(ifq) \
					-ifqLen $val(ifqlen) \
					-antType $val(ant) \
					-propInstance [new $val(prop)] \
					-phyType $val(netif) \
					-channel [new $val(chan)] \
					-topoInstance $topo \
					-wiredRouting ON \
					-agentTrace ON \
					-routerTrace OFF \
					-macTrace OFF \
					-phyTrace OFF \
					-movementTrace OFF \
					-OutgoingErrProc UniformErr \
					-eotTrace OFF \
					-toraDebug OFF

#					-OutgoingErrProc UniformErr \

proc UniformErr {} {
	set err [new ErrorModel]
	$err unit packet
	$err set rate_ 0.3
	return $err
}

# Create base station node
set baseStationNode(0) [$ns_ node 1.0.0]
$baseStationNode(0) random-motion 0

# Provide fixed coordinates for the base station node
$baseStationNode(0) set X_ 250.0
$baseStationNode(0) set Y_ 250.0
$baseStationNode(0) set Z_ 0.0
$baseStationNode(0) label "Base Station DTN"
$baseStationNode(0) shape hexagon

$ns_ duplex-link $wiredNode(0) $wiredNode(1) 3Mb 100ms SFQ
$ns_ duplex-link $wiredNode(0) $wiredNode(2) 3Mb 100ms SFQ
$ns_ duplex-link $wiredNode(0) $wiredNode(3) 3Mb 100ms SFQ
$ns_ duplex-link $wiredNode(1) $baseStationNode(0) 3Mb 100ms SFQ
$ns_ duplex-link $wiredNode(2) $baseStationNode(0) 3Mb 100ms SFQ
$ns_ duplex-link $wiredNode(3) $baseStationNode(0) 3Mb 100ms SFQ

$ns_ duplex-link-op $wiredNode(0) $wiredNode(1) orient left-up
$ns_ duplex-link-op $wiredNode(0) $wiredNode(2) orient left
$ns_ duplex-link-op $wiredNode(0) $wiredNode(3) orient left-down
$ns_ duplex-link-op $wiredNode(1) $baseStationNode(0) orient left-down
$ns_ duplex-link-op $wiredNode(2) $baseStationNode(0) orient left
$ns_ duplex-link-op $wiredNode(3) $baseStationNode(0) orient left-up

set tracefd [open $val(tr) w]
$ns_ trace-all $tracefd

#configure for mobilenodes
$ns_ node-config	-wiredRouting OFF \
					-energyModel EnergyModel \
					-idlePower 0.805 \
					-rxPower 0.950 \
					-txPower 1.400 \
					-sleepPower 0.060 \
					-transitionPower 0.0 \
					-transitionTime 0.0 \
					-initialEnergy 1000

$ns_ node-config -agentTrace ON
set wirelessNode(0) [ $ns_ node 1.0.1 ]
$wirelessNode(0) base-station [AddrParams addr2id [$baseStationNode(0) node-addr]]
$wirelessNode(0) set X_ 250.0
$wirelessNode(0) set Y_ 252.0
$wirelessNode(0) set Z_ 0.0
$wirelessNode(0) shape hexagon
$wirelessNode(0) label "Wireless DTN Target"

$ns_ initial_node_pos $wirelessNode(0) 10

#Agent/DTN set debug_ true

set dtnSource [new Agent/DTN]
#$dtnSource set debug_ true
$ns_ attach-dtnagent $wiredNode(0) $dtnSource

set ftp0 [new Application/FTP]
$ftp0 attach-agent $dtnSource

set dtn1 [new Agent/DTN]
# $dtn1 set debug_ true
$ns_ attach-dtnagent $wiredNode(1) $dtn1

set dtn2 [new Agent/DTN]
# $dtn2 set debug_ true
$ns_ attach-dtnagent $wiredNode(2) $dtn2

set dtn3 [new Agent/DTN]
# $dtn3 set debug_ true
$ns_ attach-dtnagent $wiredNode(3) $dtn3

set dtnBS [new Agent/DTN]
# $dtnBS set debug_ true

#$dtnBS set nIsBSNode_ 1
$ns_ attach-dtnagent $baseStationNode(0) $dtnBS

set dtnSink [new Agent/DTN]
$ns_ attach-dtnagent $wirelessNode(0) $dtnSink

$ns_ connect-dtn $dtnSource $dtn1
$ns_ connect-dtn $dtnSource $dtn2
$ns_ connect-dtn $dtnSource $dtn3
$ns_ connect-dtn $dtn1 $dtnBS
$ns_ connect-dtn $dtn2 $dtnBS
$ns_ connect-dtn $dtn3 $dtnBS
$ns_ connect-dtn $dtnBS $dtnSink

Application instproc recv {nBytes} {
	global ns_
	puts "Application received $nBytes bytes at [$ns_ now]."
}

set app1 [new Application]
$app1 attach-agent $dtnSink
$app1 start

#Define a 'recv' function for the class 'Agent/Ping'
Agent/Ping instproc recv {from rtt} {
	$self instvar node_
	puts "node [$node_ id] received ping answer from \
              $from with round-trip-time $rtt ms."
}

#Create two ping agents and attach them to the nodes n0 and n2
set p0 [new Agent/Ping]
$ns_ attach-agent $baseStationNode(0) $p0

set p1 [new Agent/Ping]
$ns_ attach-agent $wirelessNode(0) $p1

#Connect the two agents
$ns_ connect $p0 $p1

#Necessary for route discovery so that no time is lost at first transfer
$ns_ at 0 "$p0 send"
$ns_ at 1.5 "$p0 send"

$ns_ at $val(mainFTP-start) "puts \"Transfer of $transferSize started at $val(mainFTP-start) sec\""
$ns_ at $val(mainFTP-start) "$ftp0 send $transferSize" ;# Send 10MB

proc WriteToTrace {contents} {
    global tracefd
	puts $tracefd $contents
}

$ns_ at $val(stop) "finish"

proc finish {} {
    global ns_ tracefd namtracefd val wirelessNode baseStationNode
	# Tell all nodes when the simulation ends
   	$ns_ at $val(stop).0 "$wirelessNode(0) reset";
	$ns_ at $val(stop).0 "$baseStationNode(0) reset";
    close $tracefd
    close $namtracefd
	# exec nam $val(namtr) &
	puts "NS EXITING..."
	exit 0
}

$ns_ run
