#include "tcp-dtn-sink.h"

static class TcpDtnSinkClass : public TclClass {
public:
	TcpDtnSinkClass() : TclClass("Agent/TCPSink/TCP-DTNSink") {}
	TclObject* create(int, const char*const*) {
		return (new TcpDtnSink(new Acker));
	}
} class_tcpdtnsink;


TcpDtnSink::TcpDtnSink(Acker* acker) : TcpSink(acker)
{
	m_pDtnAgent = NULL;
}

void TcpDtnSink::recv(Packet* pPacket, Handler* pHandler)
{
	hdr_tcp *tcph = hdr_tcp::access(pPacket);
	hdr_dtn *dtnh = hdr_dtn::access(pPacket);
	// hdr_cmn *cmnh = hdr_cmn::access(pPacket);

	Bundle inBundle;
	inBundle.HeaderDTN() 		= *dtnh;
	inBundle.TCPSequenceNo()	= tcph->seqno();

	// debug("Tcp receive (Node %d) --- BundleSeq: %d, BundleSize: %d, TcpSeq: %d, SegNo: %d, Payload: %d, LeadBytes: %d\n", 
	//	addr(), dtnh->SequenceNumber(), dtnh->BundleSize(), tcph->seqno(), dtnh->SegmentNumber(), dtnh->Payload(), dtnh->LeadingBytes());

	bundleManagerTCP_.AddBundle(inBundle);
	
	if (tcph->seqno() == acker_->Seqno() + 1) // If this is the next packet expected
	{
		int nSeqNo = tcph->seqno() + 1;
		Bundle deliveryBundle; deliveryBundle = inBundle;
		Bundle *pNextInSequenceBundle = bundleManagerTCP_.GetBundle(nSeqNo);
		// debug("Node %d: Delivery bundle LeadingBytes: %d\n", addr(), deliveryBundle.HeaderDTN().LeadingBytes());

		while (pNextInSequenceBundle)
		{
			// debug("In sequence (Node %d) --- BundleSeq: %d, TcpSeq: %d, BundleSize: %d\n", 
			//	addr(), pNextInSequenceBundle->HeaderDTN().SequenceNumber(), pNextInSequenceBundle->TCPSequenceNo(), 
			//	pNextInSequenceBundle->HeaderDTN().BundleSize());
			if (deliveryBundle.IsSameAs(*pNextInSequenceBundle) && !pNextInSequenceBundle->HeaderDTN().LeadingBytes())
			{
				deliveryBundle.HeaderDTN().Payload() += pNextInSequenceBundle->HeaderDTN().Payload();
			}
			else
			{
				if (deliveryBundle.HeaderDTN().Payload() > 0)
					m_pDtnAgent->ReceiveBundlePayload(deliveryBundle, this);
				deliveryBundle = *pNextInSequenceBundle;
			}
			nSeqNo++;
			pNextInSequenceBundle = bundleManagerTCP_.GetBundle(nSeqNo);
		}
		if (deliveryBundle.HeaderDTN().Payload() > 0)
			m_pDtnAgent->ReceiveBundlePayload(deliveryBundle, this);
				
		bundleManagerTCP_.Cleanup(nSeqNo - 1);
	}
		
	TcpSink::recv(pPacket, pHandler);
}
